<?php
/**
 * A simple PHP class for a flat-file database.
 *
 * This class is for storage and access of data in a simple,
 * flat-file (text-file) database.
 * Now updated to do away with the old "separator-based" style,
 * and utilize php's much simpler serialize().
 *
 *
 * +-----------------------------------------------------------------------+
 * |                                                                       |
 * |  Copyright (C) 2004 Markku Seguerra                                   |
 * |                                                                       |
 * |  This library is free software; you can redistribute it and/or        |
 * |  modify it under the terms of the GNU Lesser General Public           |
 * |  License as published by the Free Software Foundation; either         |
 * |  version 2.1 of the License, or (at your option) any later version.   |
 * |                                                                       |
 * |  This library is distributed in the hope that it will be useful,      |
 * |  but WITHOUT ANY WARRANTY; without even the implied warranty of       |
 * |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
 * |  Lesser General Public License for more details at:                   |
 * |      http://www.gnu.org/copyleft/lesser.html                          |
 * |                                                                       |
 * +-----------------------------------------------------------------------+
 *
 * TODO:
 *      select_filter( $results_array, $search_column, $search_value )
 *
 *
 * VERSION HISTORY:
 *
 *     0.98.1 2004-06-10  - Fixed warning with integer type for counter.
 *
 *     0.98   2004-03-19  - Code cleanup, addition of LGPL license.
 *
 *     0.97   2004-02-28  - Added order_by() function to replace
 *                        sort_by_column() which has been deprecated.
 *                      - Added join() for appending columns to a
 *                        results array from another result array
 *                        (most probably from another database/table file).
 *
 *     0.96   2004-02-25  - Changed file storage format to use PHP's much
 *                        simpler serialize() function.
 *
 *     0.95   2004-02-23  - Optimization for speed improvements.
 *
 *     0.63   2004-02-12  - Initial (internal) use.
 *
 *
 * @author Markku Seguerra <vincent_markku.seguerra@up.edu.ph>
 * @version $ class.dbtxt.php, v0.98.1 2004-06-10 19:51:13 $
 * @access public
 * @copyright Markku Seguerra
 */
class dbtxt
{

    /**
     * Sets class to defaults
     *
     * @param null
     * @return null
     * @access public
     */
    function dbtxt()
    {
        $this->_db = array();
        //$this->_counter = intval('');
        settype($this->_counter, 'integer');
        $this->_columns = array();
        $this->_rows = array();

        $this->auto_inc_column = 'id';
        $this->use_auto_inc = true;

        $this->header = '<'.'?php exit(\'Access restricted.\');'."\n".
        '// dbtxt file-based storage class'."\n".
        '// more info at http://rebelpixel.com/projects/dbtxt/'."\n".
        '?'.'>'."\n";
    }




    /**
    * Opens a database.
    *
    * @param string $filename Filename of the database.
    * @param bool $write_mode Open for read only or write-mode.
    * @return null
    * @access public
    */
    function open($filename, $write_mode = false)
    {
        $this->_filename = $filename;
        $this->_write_mode = $write_mode;

        $fp = fopen($this->_filename, 'r'); // old open() buggy for multiple open then insert because of filesize()
        $this->_db = '';
        do {
            $__data = fread($fp, 8192);
            if (strlen($__data) == 0) {
                break;
            }
            $this->_db .= $__data;
        } while(true);
        fclose($fp);

        $this->_db = substr($this->_db, strpos($this->_db, '?>')+3);
        $this->_db = unserialize($this->_db);

        $this->_counter = $this->_db['counter'];
        $this->_columns = $this->_db['columns'];
        $this->_rows = $this->_db['records'];
    }




    /**
    * Close (ands save) database.
    *
    * @param null
    * @return bool Returns true if DB file was saved successfully.
    * @access public
    */
    function close()
    {
        $__close = false;
        if ($this->_write_mode) {
            $__write['counter'] = $this->_counter;
            $__write['columns'] = $this->_columns;
            $__write['records'] = $this->_rows;
            $__str = serialize($__write);
            $__str = $this->header.$__str;

            $fp = fopen ($this->_filename,  'w');
            fwrite($fp, $__str);
            if ( fclose($fp) ) {
                $__close = true;
            }
        }
        $this->dbtxt;
        return $__close;
    }




    /**
    * Close (ands save) database.
    *
    * @param string $filename Filename of the database.
    * @param array $columns Array of the column names.
    * @param bool $overwrite Overwrite file if it already exists.
    * @param bool $write_mode Open for read only or write-mode.
    * @return bool True if file was created, false if not or file already exists.
    * @access public
    */
    function new_db($filename, $columns, $overwrite = false, $write_mode = false)
    {
        $__create = false;

        $this->_filename = $filename;
        $this->_write_mode = $write_mode;

        $__write['counter'] = 0;
        $__write['columns'] = $columns;
        $__write['records'] = array();
        $__str = serialize($__write);
        $__str = $this->header.$__str;

        if ($overwrite) {
            $fp = fopen ($this->_filename,  'w');
            fwrite($fp, $__str);
            if (fclose($fp)) {
                $__create = true;
            }
        } elseif (!file_exists($this->_filename)) {
            $fp = fopen ($this->_filename,  'w');
            fwrite($fp, $__str);
            if (fclose($fp)) {
                $__create = true;
            }
        }
        clearstatcache ();
        if (file_exists($this->_filename)) {
            $this->open($this->_filename, $write_mode);
        }

        return $__create;
    }




    /**
    * Returns all records in the database.
    *
    * @param null
    * @return array Array of all records.
    * @access public
    */
    function get_all() // sort column, sort order
    {
        return $this->_rows;
    }




    /**
    * Inserts a new record (row) in the database.
    *
    * @param array $new Associative array of new values.
    * @return bool True if inserted properly. (Not yet saved!)
    * @access public
    */
    function insert($new)
    {
        $__insert = false;
        $__before = count($this->_rows);

        foreach ($this->_columns as $column_name) {
            if (  !isset($new[$column_name])  &&  $column_name != $this->auto_inc_column  ) {
                die('At least one column missing on insert.');
            }
        }
        foreach ($this->_columns as $column_name) {
            if (  $column_name == $this->auto_inc_column  &&  !isset($new[$column_name])  &&  $this->use_auto_inc  ) {
                $this->_counter = $this->_counter + 1;
                $add[$column_name] = $this->_counter;
            } else {
                $add[$column_name] = $new[$column_name];
            }
        }
        $this->_rows[] = $add;

        $__after = count($this->_rows);
        if (  ($__after - $__before) > 0  ) { // basic error-checking for insert
            $__insert = true;
        }

        return $__insert;
    }




    /**
    * Update a record (row) in the database.
    *
    * @param string $search_column The name of the column. (id, pid, etc.)
    * @param string $search_value The value you are looking for.
    * @param array $update Associative array of new values.
    * @param bool $update_all Designates update of one or all results.
    * @return bool True if updated successfully.
    * @access public
    */
    function update($search_column, $search_value, $update, $update_all = false)
    {
        // error checking for length of $search_column, $search_value should be here

        $__updated = false;
        for ($i=0; $i < count($this->_rows); $i++) {
            if ($this->_rows[$i][$search_column] == $search_value) {
                $__updated = true;
                foreach ($update as $key=>$val) {
                    $this->_rows[$i][$key] = $val;
                }
                if (!$update_all) {
                    break;
                }
            }
        }

        return $__updated;
    }




    /**
    * Gets a record (row) from the database.
    *
    * @param string $search_column The name of the column. (id, pid, etc.)
    * @param string $search_value The value you are looking for.
    * @param bool $select_all Designates one or all results.
    * @return array Array of results.
    * @access public
    */
    function select($search_column, $search_value, $select_all = true)
    {
        // error checking for length of $search_column, $search_value should be here

        $__results = array();
        foreach ($this->_rows as $row) {
            if (  $row[$search_column] ==  $search_value ) {
                $__results[] = $row;
                if (!$select_all) {
                    break;
                }
            }
        }

        return $__results;
    }




    /**
    * Deletes a record (row) in the database.
    *
    * Currently very slow for multiple delete operations.
    * Optimization needed for heavy duty use.
    *
    * @param string $search_column The name of the column. (id, pid, etc.)
    * @param string $search_value The value you are looking for.
    * @param bool $delete_all Delete first or all matching results.
    * @return bool True if record was found and deleted.
    * @access public
    */
    function delete($search_column, $search_value, $delete_all = false)
    {
        $__deleted = false;
        $__before = count($this->_rows);

        if ($delete_all) {
            $count_matches = 0;
            foreach ($this->_rows as $key=>$val) {
                if ($val[$search_column] == $search_value) {
                    $count_matches++;
                }
            }
            for ($i=0; $i < $count_matches; $i++) {
                $this->__delete($search_column, $search_value);
            }
        } else {
            $this->__delete($search_column, $search_value);
        }

        $__after = count($this->_rows);
        if (  ($__before - $__after) > 0  ) { // basic error-checking for delete
            $__deleted = true;
        }

        return $__deleted;
    }




    /**
    * Sorts a result array by the given column. Note: This is deprecated. Use order_by().
    *
    * @param array $result_array The results array
    * @param string $search_column The name of the column. (id, pid, etc.)
    * @param string $search_value The value you are looking for.
    * @param bool $descending_order Use descending order?
    * @return array Sorted results.
    * @access public
    */
    function sort_by_column(&$result_array, $search_column, $descending_order = false)
    {
        for ($i=0; $i < count($result_array); $i++) {
            $_padded[$i] = $result_array[$i];
            $_zero['0000000_sort'] = $result_array[$i][$search_column];
            $_padded[$i] = array_merge($_zero, $_padded[$i]);
        }
        sort($_padded);
        if ($descending_order) {
            $_padded = array_reverse($_padded);
        }
        for ($i=0; $i < count($_padded); $i++) {
            unset($_padded[$i]['0000000_sort']);
        }
        $result_array = $_padded;
        return $_padded;
    }




    /**
    * Sorts a result array by the given column.
    *
    * @param array $result_array The results array
    * @param string $search_column The name of the column. (id, pid, etc.)
    * @param string $search_value The value you are looking for.
    * @param string $order Use descending order?
    * @return array Sorted results.
    * @access public
    */
    function order_by($result_array, $search_column, $order = 'ASC')
    {
        switch ($order) {
            case 'ASC': $descending_order = false;
                        break;
            case 'DESC': $descending_order = true;
                        break;
            case 1: $descending_order = true;
                        break;
            case 0: $descending_order = false;
                        break;
            default: $descending_order = false;
                        break;
        }
        for ($i=0; $i < count($result_array); $i++) {
            $_padded[$i] = $result_array[$i];
            $_zero['0000000_sort'] = $result_array[$i][$search_column];
            $_padded[$i] = array_merge($_zero, $_padded[$i]);
        }
        if (count($_padded) > 1) {
            sort($_padded);
            if ($descending_order) {
                $_padded = array_reverse($_padded);
            }
        }
        for ($i=0; $i < count($_padded); $i++) {
            unset($_padded[$i]['0000000_sort']);
        }
        return $_padded;
    }




    /**
    * Joins two result arrays, merging the columns if match is found.
    *
    * @param array $main_array The main results array
    * @param array $join_array The array containing values to be appended to the main array
    * @param string $search_column_main The name of the column in the main array. (id, pid, etc.)
    * @param string $search_column_join The name of the column in the second array. (id, pid, etc.)
    * @param string $append The string to be appended to a merged key.
    * @return array The merged array with appended keys and values.
    * @access public
    */
    function join($main_array, $join_array, $search_column_main, $search_column_join, $append = '__')
    {
        if (  !is_array($main_array)  || !is_array($join_array)  ) {
            return false;
        }

        $__results = array();
        foreach ($main_array as $__1_key=>$__1_value) {
            $__found = false;
            foreach ($join_array as $__2_key=>$__2_value) {
                if ($__1_value[$search_column_main] == $__2_value[$search_column_join]) {
                    $__found = true;
                    $__2_value = $this->__pre_append($__2_value, $append);
                    $__results[] = array_merge($__1_value, $__2_value);
                    break;
                }
            }
            if (!$__found) {
                $__results[] = $__1_value;
            }
        }

        return $__results;
    }




    /**
    * Get an incremented counter ID for an "id" field. Note: This is deprecated.
    *
    * @param null
    * @return integer The generated ID.
    * @access public
    */
    function new_id()
    {
        $this->_counter = $this->_counter + 1;
        return $this->_counter;
    }




    /**
    * Deletes the first matched record (row) in the database.
    *
    * @param string $search_column The name of the column. (id, pid, etc.)
    * @param string $search_value The value you are looking for.
    * @return null
    * @access private
    */
    function __delete($search_column, $search_value)
    {
        foreach ($this->_rows as $key=>$val) {
            if ($val[$search_column] == $search_value) {
                $__cut = $key;
                break;
            }
        }
        if (  isset($__cut)  ) {
            $__db_pre = array_slice($this->_rows, 0, $__cut);
            $__db_suf = array_slice($this->_rows, $__cut + 1);
            $this->_rows = array_merge($__db_pre, $__db_suf);
        }
    }




    /**
    * Appends the "append string" to a merged column.
    *
    * @param array $array_row A single results record.
    * @param string $append The "append string."
    * @return array The merged record.
    * @access private
    */
    function __pre_append($array_row, $append)
    {
        $__result = array();
        foreach ($array_row as $key=>$value) {
            $__result[$append.$key] = $value;
        }
        return $__result;
    }




}
?>